#include <windows.h>

//
// HOW TO MAKE A PLUGIN:
//
// (1) Create a .DLL library
// (2) Export some functions
// (3) Rename your .DLL file to .CL one
//
// Put .CL file into folder with MSVC_Plugin.dll file. You can create cubfolder(s) if needed.
//
// Your .CL library will be called when compiler is executed
// (it does not called when MS Developer studio is loaded)
//
// Contact support@softcab.com if you need assistance.
//


//
// we are going to export three functions
//

#define _DLL_ __declspec(dllexport)

//
// Definitions of exported functions
//

typedef void  (WINAPI *FNLOAD)();
typedef void  (WINAPI *FNUNLOAD)();
typedef void  (WINAPI *FNCMDLINE)(const char *line);
typedef DWORD (WINAPI *LPFNGET)(char *mem, DWORD mem_size, const char *tag);
typedef void  (WINAPI *FNFILE)(char *fnTemp, const char *fnOriginal);

//
// Call GetEnvironmentVariable() in order to get the following:
//
//	Variable				Description
//	SOFTCAB_INI				full path to .CFG file name
//	SOFTCAB_SECT			default section name in .CFG file (use SOFTCAB_DSP (call GetShortPath first), if not specified)
//	SOFTCAB_DSP				full path to .DSP file of current ptoject
//	SOFTCAB_DSP_PATH		full path to the directory where .DSP file is located
//	SOFTCAB_NAME			name of current project
//	SOFTCAB_TYPE			type of current ptoject (should be "Build" in most cases)
//	SOFTCAB_CONF			configuration name (something like "My Project - Release")
//	SOFTCAB_NUM				build counter for current session. Increments every other build
//	SOFTCAB_STUDIO_NAME		MSDEV name (typically 'Microsoft Visial Studio')
//	SOFTCAB_STUDIO_VER		MSDEV version number (typically '5,0', '6'0' or '7.0')
//	SOFTCAB_STUDIO_PATH		Path to folder with MSDEV.EXE file
//	SOFTCAB_STUDIO_CURDIR	Current working directory (obviously, the same as SOFTCAB_DSP_PATH)
//	SOFTCAB_STUDIO_EXE		Full path to MSDEV.EXE file
//	SOFTCAB_CPP_FILE		Full path to current C/CPP file
//	SOFTCAB_CPP_LINE		Line number in current C/CPP file
//	SOFTCAB_H_FILE			Full path to header file. If C/CPP file being parsed, this variable is the same as SOFTCAB_CPP_FILE
//	SOFTCAB_H_LINE			Line number in .H file. If C/CPP file parsed, it's the same as SOFTCAB_CPP_LINE
//  SOFTCAB_TEMP			Program's temporary directory
//

HANDLE hInstance = NULL;

//
// DLL Entry
//

BOOL APIENTRY DllMain(HANDLE hModule, DWORD  dwReason, LPVOID lpReserved)
{	hInstance = (HINSTANCE)hModule;
    return TRUE;
}

//
// This is called right after loading plugin
//

_DLL_ void WINAPI Load()
{	OutputDebugString("*Plugin*  Load()\r\n");
}

//
// This is called before unloading plugin
//

_DLL_ void WINAPI Unload()
{	OutputDebugString("*Plugin*  Unload()\r\n");
}

//
// This must return the text corresponding to param
// param - the string that comes right after question mark in source tag: <{RunDll:yourfile.dll?PARAM HERE}>
// Function must return anount of bytes actually copied into mem, including concluding zero
// Note, this functin is called twice - first time with mem and mem_size set to zero, so main
// program can determine how large memory to allocate.
//

_DLL_ DWORD WINAPI onTag(char *mem, DWORD mem_size, const char *tag)
{	// printing debugging information
	char s[2048];
	wsprintf(s, "*Plugin* onTag(0x%X, %lu, \"%s\")\r\n", mem, mem_size, tag);
	OutputDebugString(s);

	if(mem)
	{	lstrcpyn(mem, "MessageBox(NULL, \"Hello, world!\", \"Test\", MB_OK);", mem_size);
	}
	// it's Ok to return larger memory, than needed
	return 512;
}

//
// you can analyse command line here.
// This may (and certainly will be) called more than once per session.
//

_DLL_ void WINAPI onCmdLine(const char *command_line)
{	char s[2048];
	wsprintf(s, "*Plugin* onCmdLine(%s)\r\n", command_line);
	OutputDebugString(s);
}

//
// This allows to insert any text right after function definition and before function body.
// In other words, the text will be inserted:    int my_strlen(char *str) #HERE#{ return strlen(str); }
// Function must return how many bytes it needs to store result text
//

_DLL_ DWORD WINAPI onFuncBegin(char *mem, DWORD mem_size, const char *function)
{	// printing debugging information
	char s[2048];
	wsprintf(s, "*Plugin* onFuncBegin(0x%X, %lu, \"%s\")\r\n", mem, mem_size, function);
	OutputDebugString(s);

	if(mem)
	{	wsprintf(mem, "{ TRACE(\"%s()\\r\\n\");", function);
	}
	// it's Ok to return larger memory, than needed
	return 512;
}

//
// This allows to insert any text right after function body.
// In other words, the text will be inserted:    int my_strlen(char *str) { return strlen(str); }#HERE#
//

_DLL_ DWORD WINAPI onFuncEnd(char *mem, DWORD mem_size, const char *function)
{	// printing debugging information
	char s[2048];
	wsprintf(s, "*Plugin* onFuncEnd(0x%X, %lu, \"%s\")\r\n", mem, mem_size, function);
	OutputDebugString(s);

	if(mem)
	{	lstrcpyn(mem, "}", mem_size);
	}
	// it's Ok to return larger memory, than needed
	return 512;
}

//
// This is called when source file parsing is done.
// fnTemp     - modified file
// fnOriginal - original file
//

_DLL_ void WINAPI onFile(char *fnTemp, const char *fnOriginal)
{	char s[2048];
	wsprintf(s, "*Plugin* onFile(\"%s\", \"%s\")\r\n", fnTemp, fnOriginal);
	OutputDebugString(s);
}

//
// The following staff will decrease executable size
//

#ifndef _DEBUG
#pragma comment(linker, "/MERGE:.rdata=.text")
#pragma comment(linker, "/FILEALIGN:512")
#pragma comment(linker, "/SECTION:.text,EWRX")
#pragma comment(linker, "/IGNORE:4078")
#pragma comment(linker, "/ENTRY:DllMain")
#pragma comment(linker, "/subsystem:windows")
#pragma comment(linker, "/DLL")
#pragma comment(linker, "/DEF:plugin.def")
#pragma comment(lib,"kernel32.lib")
#pragma comment(lib,"user32.lib")
#endif
